import os
import unittest


from vsg import vhdlFile
from vsg import rule_list
from vsg.tests import utils

oTimestamp = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'timestamp.vhdl'))
oSpiSlave = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'spi_slave.vhd'))
oSpiMaster = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'spi_master.vhd'))
oGrpDebouncer = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'grp_debouncer.vhd'))
oPIC = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'PIC.vhd'))

class testVhdlFileMethods(unittest.TestCase):

    def test_timestamp_vhdl(self):
        oRuleList = rule_list.rule_list(oTimestamp)
        oRuleList.fix(7)
        lExpected = ['']
        utils.read_file(os.path.join(os.path.dirname(__file__),'timestamp.fixed.vhdl'), lExpected)
        for iLineNumber, sLine in enumerate(lExpected):
            self.assertEqual(oTimestamp.lines[iLineNumber].line, sLine)

    def test_spi_slave(self):
        oRuleList = rule_list.rule_list(oSpiSlave)
        oRuleList.fix(7)
        lExpected = ['']
        utils.read_file(os.path.join(os.path.dirname(__file__),'spi_slave.fixed.vhd'), lExpected)
        for iLineNumber, sLine in enumerate(lExpected):
            self.assertEqual(oSpiSlave.lines[iLineNumber].line, sLine)

    def test_spi_master(self):
        oRuleList = rule_list.rule_list(oSpiMaster)
        oRuleList.fix(7)
        lExpected = ['']
        utils.read_file(os.path.join(os.path.dirname(__file__),'spi_master.fixed.vhd'), lExpected)
        for iLineNumber, sLine in enumerate(lExpected):
            self.assertEqual(oSpiMaster.lines[iLineNumber].line, sLine)

    def test_grp_debouncer(self):
        oRuleList = rule_list.rule_list(oGrpDebouncer)
        oRuleList.fix(7)
        lExpected = ['']
        utils.read_file(os.path.join(os.path.dirname(__file__),'grp_debouncer.fixed.vhd'), lExpected)
        for iLineNumber, sLine in enumerate(lExpected):
            self.assertEqual(oGrpDebouncer.lines[iLineNumber].line, sLine)

    def test_pic(self):
        oRuleList = rule_list.rule_list(oPIC)
        oRuleList.fix(7)
        lExpected = ['']
        utils.read_file(os.path.join(os.path.dirname(__file__),'PIC.fixed.vhd'), lExpected)
        for iLineNumber, sLine in enumerate(lExpected):
            self.assertEqual(oPIC.lines[iLineNumber].line, sLine)



import os

import unittest

from vsg.rules import procedure
from vsg import vhdlFile

# Read in test file used for all tests
oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'procedure_test_input.vhd'))


class testRuleProcedureMethods(unittest.TestCase):

    def test_rule_001(self):
        oRule = procedure.rule_001()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'procedure')
        self.assertEqual(oRule.identifier, '001')
        dExpected = [14,16,40]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_002(self):
        oRule = procedure.rule_002()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'procedure')
        self.assertEqual(oRule.identifier, '002')
        dExpected = [41]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_003(self):
        oRule = procedure.rule_003()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'procedure')
        self.assertEqual(oRule.identifier, '003')
        dExpected = [14,42]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_004(self):
        oRule = procedure.rule_004()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'procedure')
        self.assertEqual(oRule.identifier, '004')
        dExpected = [17,18,19,20,45,46,47,48]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_005(self):
        oRule = procedure.rule_005()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'procedure')
        self.assertEqual(oRule.identifier, '005')
        dExpected = [89,90]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)





import os
import unittest

from vsg.rules import type_definition
from vsg import vhdlFile

# Read in test file used for all tests
oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'type_test_input.vhd'))


class testFixRuleSignalMethods(unittest.TestCase):

    def test_fix_rule_001(self):
        oRule = type_definition.rule_001()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_002(self):
        oRule = type_definition.rule_002()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_003(self):
        oRule = type_definition.rule_003()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_004(self):
        oRule = type_definition.rule_004()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_005(self):
        oRule = type_definition.rule_005()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_006(self):
        oRule = type_definition.rule_006()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_007(self):
        oRule = type_definition.rule_007()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_008(self):
        oRule = type_definition.rule_008()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_009(self):
        oRule = type_definition.rule_009()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)
        self.assertEqual(oFile.lines[6].line, '  type a is (')
        self.assertEqual(oFile.lines[7].line, ' B, C,')

    def test_fix_rule_010(self):
        oRule = type_definition.rule_010()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_011(self):
        oRule = type_definition.rule_011()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)



import os

import unittest
from vsg import vhdlFile

oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','type_definition','type_test_input.vhd'))

class testVhdlFileTypeAssignments(unittest.TestCase):


    def test_isTypeKeyword(self):
        lExpected = [4,6,11,13,27,29,34,36,43,54,57,69]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isTypeKeyword:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isTypeEnd(self):
        lExpected = [4,9,11,19,27,32,34,40,48,55,62,75]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isTypeEnd:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isTypeEnumeratedKeyword(self):
        lExpected = [4,6,27,29,36,43]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isTypeEnumeratedKeyword:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isTypeEnumeratedEnd(self):
        lExpected = [4,9,27,32,40,48]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isTypeEnumeratedEnd:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_insideTypeEnumerated(self):
        lExpected = [4,6,7,8,9,27,29,30,31,32,36,37,38,39,40,43,44,45,46,47,48]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.insideTypeEnumerated:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_TypeIndent(self):
        #           [   0,   1,2,   3,4,   5,6,7,8,9,  10,11,  12,13,14,15,16,17,  18,19,   20]
        lExpected = [None,None,0,None,1,None,1,2,2,2,None, 1,None, 1, 2, 2, 2, 2,None, 1, None]
        # Generic actual list
        lActual = []
        iMaxCheck = len(lExpected)
        for iIndex, oLine in enumerate(oFile.lines):
            if iIndex == iMaxCheck:
                break
            lActual.append(oLine.indentLevel)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isSubtypeKeyword(self):
        lExpected = [51,65]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isSubtypeKeyword:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isSubtypeEnd(self):
        lExpected = [51,66]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isSubtypeEnd:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_insideSubtype(self):
        lExpected = [51,65,66]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.insideSubtype:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isTypeArrayKeyword(self):
        lExpected = [54]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isTypeArrayKeyword:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isTypeArrayEnd(self):
        lExpected = [55]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isTypeArrayEnd:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_insideTypeArray(self):
        lExpected = [54,55]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.insideTypeArray:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)


    def test_isTypeRecordKeyword(self):
        lExpected = [13,57,70]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isTypeRecordKeyword:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isTypeRecordEnd(self):
        lExpected = [19,62,75]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isTypeRecordEnd:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_insideTypeRecord(self):
        lExpected = [13,14,15,16,17,18,19,57,58,59,60,61,62,70,71,72,73,74,75]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.insideTypeRecord:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isEndArchitecture(self):
        lExpected = [23]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isEndArchitecture:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)



import os

import unittest

from vsg.rules import process
from vsg import vhdlFile

# Read in test file used for all tests
oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','process','process_test_input.vhd'))
oFileEvent = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'process_event_test_input.vhd'))

class testRuleProcessMethods(unittest.TestCase):

    def test_rule_001(self):
        oRule = process.rule_001()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'process')
        self.assertEqual(oRule.identifier, '001')
        dExpected = [17,24,32,38,46]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_002(self):
        oRule = process.rule_002()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'process')
        self.assertEqual(oRule.identifier, '002')
        dExpected = [17,24]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_003(self):
        oRule = process.rule_003()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'process')
        self.assertEqual(oRule.identifier, '003')
        dExpected = [20,40]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_004(self):
        oRule = process.rule_004()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'process')
        self.assertEqual(oRule.identifier, '004')
        dExpected = [20,28]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_005(self):
        oRule = process.rule_005()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'process')
        self.assertEqual(oRule.identifier, '005')
        dExpected = [17,24,32]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_006(self):
        oRule = process.rule_006()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'process')
        self.assertEqual(oRule.identifier, '006')
        dExpected = [9,15,22,30]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_007(self):
        oRule = process.rule_007()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'process')
        self.assertEqual(oRule.identifier, '007')
        dExpected = [15,30]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_008(self):
        oRule = process.rule_008()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'process')
        self.assertEqual(oRule.identifier, '008')
        dExpected = [15,36]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_009(self):
        oRule = process.rule_009()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'process')
        self.assertEqual(oRule.identifier, '009')
        dExpected = [22,42]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_010(self):
        oRule = process.rule_010()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'process')
        self.assertEqual(oRule.identifier, '010')
        dExpected = [6]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_011(self):
        oRule = process.rule_011()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'process')
        self.assertEqual(oRule.identifier, '011')
        dExpected = [42]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_012(self):
        oRule = process.rule_012()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'process')
        self.assertEqual(oRule.identifier, '012')
        dExpected = [51,57]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_013(self):
        oRule = process.rule_013()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'process')
        self.assertEqual(oRule.identifier, '013')
        dExpected = [6,27,33]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_014(self):
        oRule = process.rule_014()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'process')
        self.assertEqual(oRule.identifier, '014')
        dExpected = [19,33,39]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_015(self):
        oRule = process.rule_015()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'process')
        self.assertEqual(oRule.identifier, '015')
        dExpected = [63]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_016(self):
        oRule = process.rule_016()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'process')
        self.assertEqual(oRule.identifier, '016')
        dExpected = [6,11,17,24,32,38,51,55]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_017(self):
        oRule = process.rule_017()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'process')
        self.assertEqual(oRule.identifier, '017')
        dExpected = [46]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_018(self):
        oRule = process.rule_018()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'process')
        self.assertEqual(oRule.identifier, '018')
        dExpected = [9,15,22,30,36,42,53,60]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_019(self):
        oRule = process.rule_019()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'process')
        self.assertEqual(oRule.identifier, '019')
        dExpected = [48]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_020(self):
        oRule = process.rule_020()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'process')
        self.assertEqual(oRule.identifier, '020')
        dExpected = [19,25,26]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_021(self):
        oRule = process.rule_021()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'process')
        self.assertEqual(oRule.identifier, '021')
        dExpected = [59,70]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_022(self):
        oRule = process.rule_022()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'process')
        self.assertEqual(oRule.identifier, '022')
        dExpected = [47,52,59,64]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_023(self):
        oRule = process.rule_023()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'process')
        self.assertEqual(oRule.identifier, '023')
        dExpected = [9,48,53,60,65]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_024(self):
        oRule = process.rule_024()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'process')
        self.assertEqual(oRule.identifier, '024')
        dExpected = [63]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_025(self):
        oRule = process.rule_025()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'process')
        self.assertEqual(oRule.identifier, '025')
        dExpected = [68]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_026(self):
        oRule = process.rule_026()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'process')
        self.assertEqual(oRule.identifier, '026')
        dExpected = [75]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_027(self):
        oRule = process.rule_027()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'process')
        self.assertEqual(oRule.identifier, '027')
        dExpected = [77,84]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_028(self):
        oRule = process.rule_028()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'process')
        self.assertEqual(oRule.identifier, '028')
        dExpected = [27,33]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_029(self):
        oRule = process.rule_029()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'process')
        self.assertEqual(oRule.identifier, '029')
        dExpected = [9,13]
        oRule.analyze(oFileEvent)
        self.assertEqual(oRule.violations, dExpected)



import os

import unittest
from vsg import vhdlFile

oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','instantiation','instantiation_test_input.vhd'))
oFileGeneric = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','instantiation','instantiation_generic_test_input.vhd'))
oFileGenerate = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','instantiation','instantiation_generate_test_input.vhd'))
oFileDirect = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','instantiation','instantiation_direct_test_input.vhd'))

class testVhdlFileInstantiationAssignments(unittest.TestCase):


    def test_isInstantiation_assignment(self):
        lExpected = [6,17,23,29,36,44,52,57,63,69,75,82,91]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isInstantiationDeclaration:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_insideInstantiation_assignment(self):
        lExpected = [6,7,8,9,10,11,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,44,45,46,47,48,49,50,52,53,54,55,57,58,59,60,61,63,64,65,66,69,70,71,72,73,75,76,77,78,79,80,82,83,84,85,86,87,88,89,91,92,93,94,95,96]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.insideInstantiation:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isInstantiationPortKeyword_assignment(self):
        lExpected = [7,18,24,31,37,45,52,58,64,70,76,83,92]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isInstantiationPortKeyword:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isInstantiationPortEnd_assignment(self):
        lExpected = [11,22,28,35,42,50,55,61,66,73,80,89,96]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isInstantiationPortEnd:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isInstantiationPortAssignment_assignment(self):
        lExpected = [8,9,10,19,20,21,25,26,27,32,33,34,39,41,46,47,48,53,54,55,58,59,60,65,71,72,77,78,79,85,86,87,88,94,95]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isInstantiationPortAssignment:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isInstantiationGenericKeyword_assignment(self):
        lExpected = [7,22,33,44,54,63,73,81]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFileGeneric.lines):
            if oLine.isInstantiationGenericKeyword:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isInstantiationGenericEnd_assignment(self):
        lExpected = [10,25,36,46,56,65,73,85]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFileGeneric.lines):
            if oLine.isInstantiationGenericEnd:
                lActual.append(iIndex)
        # Compare
#        utils.print_attributes(oFileGeneric.lines[81])
        self.assertEqual(lActual, lExpected)

    def test_isInstantiationGenericAssignment_assignment(self):
        lExpected = [8,9,23,24,34,35,45,46,54,55,64,65,83,84]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFileGeneric.lines):
            if oLine.isInstantiationGenericAssignment:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isInstantiationIdention_assignment(self):
        self.maxDiff = None
        #lExpected = [ 6, 7, 8, 9,10,11,12,13,14,15,21,22,23,24,25,26,27,28,29,30,32,33,34,35,36,37,38,39,40,41,43,44,45,46,47,48,49,50,51,53,54,55,56,57,58,59,60,61,63,64,65,66,67,68,69,70,72,73,74,75,76,77,78,80,81,  82,83,84,85,86,  87,88,89,90,91]
        lExpected =  [ 1, 2, 3, 3, 2, 2, 3, 3, 3, 2, 1, 2, 3, 3, 2, 2, 3, 3, 3, 2, 1, 2, 3, 3, 2, 2, 3, 3, 3, 2, 1, 2, 3, 3, 2, 3, 3, 3, 2, 1, 2, 3, 2, 2, 3, 3, 3, 2, 1, 3, 3, 2, 3, 3, 3, 2, 1, 2, 2, 3, 3, 3, 2, 1, 2,None, 3, 3, 2, 2,None, 3, 3, 3, 2]
        # Generic actual list
        lActual = []
        for oLine in oFileGeneric.lines:
            if oLine.insideInstantiation:
                lActual.append(oLine.indentLevel)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isInstantiation_assignment_inside_generate(self):
        lExpected = [8]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFileGenerate.lines):
            if oLine.isInstantiationDeclaration:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isDirectInstantiation_assignment_inside_generate(self):
        lExpected = [6,13,20]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFileDirect.lines):
            if oLine.isDirectInstantiationDeclaration:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

        lExpected = [27]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFileDirect.lines):
            if oLine.isInstantiationDeclaration:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)



import os
import unittest

from vsg import rule
from vsg import check
from vsg import line
from vsg import vhdlFile

sFileName = os.path.join(os.path.dirname(__file__),'check_test_input.txt')


class testCheckFunctions(unittest.TestCase):

    def test_is_no_blank_line_after(self):
        oFile = vhdlFile.vhdlFile(sFileName)
        oFile.lines.append(line.line('Simple line'))
        oFile.lines.append(line.blank_line())

        oRule = no_blank_check_rule()
        self.assertEqual(oRule.violations, [])
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [1])

    def test_get_package_name(self):
        self.assertEqual(check.get_package_name(line.line('end package FIFO;')), 'FIFO')
        self.assertEqual(check.get_package_name(line.line('end package;')), '')
        self.assertEqual(check.get_package_name(line.line('end FIFO;')), '')
        self.assertEqual(check.get_package_name(line.line('end;')), '')
        self.assertEqual(check.get_package_name(line.line('package FIFO_PKG is')), 'FIFO_PKG')


class no_blank_check_rule(rule.rule):

    def __init__(self):
        rule.rule.__init__(self, 'debug', '001')
        self.solution = None
        self.phase = None

    def analyze(self, oFile):
        check.is_no_blank_line_after(self, oFile, 1)



import os
import unittest

from vsg.rules import generate
from vsg import vhdlFile

# Read in test file used for all tests
oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','generate','generate_test_input.vhd'))

class testRuleGenerateMethods(unittest.TestCase):

    def test_rule_001(self):
        oRule = generate.rule_001()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'generate')
        self.assertEqual(oRule.identifier, '001')

        dExpected = [11,16]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_002(self):
        oRule = generate.rule_002()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'generate')
        self.assertEqual(oRule.identifier, '002')

        dExpected = [21,26]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_003(self):
        oRule = generate.rule_003()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'generate')
        self.assertEqual(oRule.identifier, '003')

        dExpected = [54]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_004(self):
        oRule = generate.rule_004()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'generate')
        self.assertEqual(oRule.identifier, '004')

        dExpected = [56]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_005(self):
        oRule = generate.rule_005()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'generate')
        self.assertEqual(oRule.identifier, '005')

        dExpected = [11,16]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_006(self):
        oRule = generate.rule_006()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'generate')
        self.assertEqual(oRule.identifier, '006')

        dExpected = [12,17]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_007(self):
        oRule = generate.rule_007()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'generate')
        self.assertEqual(oRule.identifier, '007')

        dExpected = [14,19]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)




import os

import unittest

from vsg.rules import instantiation
from vsg import vhdlFile

oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','instantiation','instantiation_test_input.vhd'))
oFileGeneric = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','instantiation','instantiation_generic_test_input.vhd'))
oFileComment = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'instantiation_comment_test_input.vhd'))
oFileDirect = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'instantiation_direct_test_input.vhd'))

class testFixRuleInstantiationMethods(unittest.TestCase):

    def test_fix_rule_001(self):
        oRule = instantiation.rule_001()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_002(self):
        oRule = instantiation.rule_002()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_003(self):
        oRule = instantiation.rule_003()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_004(self):
        oRule = instantiation.rule_004()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_005(self):
        oRule = instantiation.rule_005()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_006(self):
        oRule = instantiation.rule_006()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_007(self):
        oRule = instantiation.rule_007()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_008(self):
        oRule = instantiation.rule_008()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_009(self):
        oRule = instantiation.rule_009()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_010(self):
        oRule = instantiation.rule_010()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_011(self):
        oRule = instantiation.rule_011()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])
        # Note: the line below can change if this test is ran individually
        self.assertEqual(oFile.lines[76].line, '      PORT_1(c_index)     => w_port_1,')

    def test_fix_rule_012(self):
        oRule = instantiation.rule_012()
        oRule.fix(oFileGeneric)
        oRule.analyze(oFileGeneric)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_013(self):
        oRule = instantiation.rule_013()
        oRule.fix(oFileGeneric)
        oRule.analyze(oFileGeneric)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_014(self):
        oRule = instantiation.rule_014()
        oRule.fix(oFileGeneric)
        oRule.analyze(oFileGeneric)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_015(self):
        oRule = instantiation.rule_015()
        oRule.fix(oFileGeneric)
        oRule.analyze(oFileGeneric)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_016(self):
        oRule = instantiation.rule_016()
        oRule.fix(oFileGeneric)
        oRule.analyze(oFileGeneric)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_017(self):
        oRule = instantiation.rule_017()
        oRule.fix(oFileGeneric)
        oRule.analyze(oFileGeneric)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_001_generics(self):
        oRule = instantiation.rule_001()
        oRule.fix(oFileGeneric)
        oRule.analyze(oFileGeneric)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_018(self):
        oRule = instantiation.rule_018()
        oRule.fix(oFileGeneric)
        oRule.analyze(oFileGeneric)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_019(self):
        oRule = instantiation.rule_019()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_020(self):
        oRule = instantiation.rule_020()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])
        self.assertEqual(oFile.lines[64].indentLevel, oFile.lines[63].indentLevel + 1)

    def test_fix_rule_021(self):
        oRule = instantiation.rule_021()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])
        self.assertEqual(oFile.lines[85].line,'      PORT_1 => w_port_1,')
        self.assertEqual(oFile.lines[86].line,' PORT_2 => w_port_2,')
        self.assertEqual(oFile.lines[87].line,'      PORT_3 => w_port_3,')

    def test_fix_rule_022(self):
        oRule = instantiation.rule_022()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_023(self):
        oRule = instantiation.rule_023()
        oRule.fix(oFileComment)
        oRule.analyze(oFileComment)
        self.assertEqual(oRule.violations, [])
        self.assertEqual(oFileComment.lines[24].line,'      generic_1 => \'0\',')
        self.assertFalse(oFileComment.lines[24].hasComment)
        self.assertFalse(oFileComment.lines[24].hasInlineComment)
        self.assertEqual(oFileComment.lines[29].line,'      port_2 => \'1\',')
        self.assertFalse(oFileComment.lines[29].hasComment)
        self.assertFalse(oFileComment.lines[29].hasInlineComment)
        self.assertEqual(oFileComment.lines[31].line,'      port_4 => \'1\'')
        self.assertFalse(oFileComment.lines[31].hasComment)
        self.assertFalse(oFileComment.lines[31].hasInlineComment)

    def test_fix_fule_025(self):
        oRule = instantiation.rule_025()
        oRule.fix(oFileGeneric)
        oRule.analyze(oFileGeneric)
        self.assertEqual(oRule.violations, [])
        self.assertEqual(oFileGeneric.lines[81].line, '    generic map (')
        self.assertEqual(oFileGeneric.lines[82].line, '      GENERIC_1 => generic_1,')

    def test_fix_fule_026(self):
        oRule = instantiation.rule_026()
        oRule.fix(oFileGeneric)
        oRule.analyze(oFileGeneric)
        self.assertEqual(oRule.violations, [])
        self.assertEqual(oFileGeneric.lines[85].line, '    port map (')
        self.assertEqual(oFileGeneric.lines[86].line, '      PORT_1 => w_port_1,')

    def test_fix_fule_027(self):
        oRule = instantiation.rule_027()
        oRule.fix(oFileDirect)
        oRule.analyze(oFileDirect)
        self.assertEqual(oRule.violations, [])
        self.assertEqual(oFileDirect.lines[13].line, '  U_INST1 : entity library.INST1')
        self.assertEqual(oFileDirect.lines[13].line, '  U_INST1 : entity library.INST1')

    def test_fix_fule_028(self):
        oRule = instantiation.rule_028()
        oRule.fix(oFileDirect)
        oRule.analyze(oFileDirect)
#        self.assertEqual(oRule.violations, [])
        self.assertEqual(oFileDirect.lines[20].line, '  U_INST1 : entity library.INST1')




import os

import unittest

from vsg.rules import library
from vsg import vhdlFile


# Read in test file used for all tests
oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','library','library_test_input.vhd'))

class testFixRuleLibraryMethods(unittest.TestCase):

    def test_fix_rule_001(self):
        oRule = library.rule_001()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_002(self):
        oRule = library.rule_002()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_003(self):
        oRule = library.rule_003()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_004(self):
        oRule = library.rule_004()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_005(self):
        oRule = library.rule_005()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_006(self):
        oRule = library.rule_006()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_007(self):
        oRule = library.rule_007()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_008(self):
        oRule = library.rule_008()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])




from rule_001 import rule_001
from rule_002 import rule_002
from rule_003 import rule_003
from rule_004 import rule_004
from rule_005 import rule_005
from rule_006 import rule_006
from rule_007 import rule_007
from rule_008 import rule_008
from rule_009 import rule_009
from rule_010 import rule_010
from rule_011 import rule_011
from rule_012 import rule_012
from rule_013 import rule_013



import os

import unittest
from vsg import vhdlFile

oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','case','case_test_input.vhd'))


class testVhdlFileIfAssignments(unittest.TestCase):

    def test_isCaseKeyword_assignment(self):
        lExpected = [9,41,77]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isCaseKeyword:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isCaseIsKeyword_assignment(self):
        lExpected = [9,43,77]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isCaseIsKeyword:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isCaseWhenKeyword_assignment(self):
        lExpected = [11,17,23,29,45,52,58,66]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isCaseWhenKeyword:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isEndCaseKeyword_assignment(self):
        lExpected = [33,70,79]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isEndCaseKeyword:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_insideCaseWhen_assignment(self):
        lExpected = [11,17,23,29,45,46,52,58,59,60,66]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.insideCaseWhen:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isCaseWhenEnd_assignment(self):
        lExpected = [11,17,23,29,46,52,60,66]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isCaseWhenEnd:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_caseIndent_assignment(self):
        self.maxDiff = None
#       lExpected = [   0,   1,2,   3,4,   5,6,7,8,9,  10,11,  12,13,14,15,  16,17,  18,19,20,21,  22,23,  24,25,26,27,  28,29, 30,31,  32,33,34,35,  36]
        lExpected = [None,None,0,None,0,None,1,1,2,2,None, 3,None, 4, 4, 4,None, 3,None, 4, 4, 4,None, 3,None, 4, 4, 4,None,3,None, 4,None, 2, 2, 1,None]
        # Generic actual list
        lActual = []
        iMaxCheck = len(lExpected)
        for iIndex, oLine in enumerate(oFile.lines):
            if iIndex == iMaxCheck:
                break
            lActual.append(oLine.indentLevel)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isCaseNull_assignment(self):
        lExpected = [31,68]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isCaseNull:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)



import os

import unittest
import sys
sys.path.append('vsg')

from vsg.rules import whitespace
from vsg import vhdlFile
from vsg import line

sFileName = os.path.join(os.path.dirname(__file__),'whitespace_test_input.txt')


class testFixRuleWhitespaceMethods(unittest.TestCase):

    def test_fix_001(self):
        oRule = whitespace.rule_001()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'whitespace')
        self.assertEqual(oRule.identifier, '001')

        oFile = vhdlFile.vhdlFile(sFileName)

        dExpected = []
        oFile.lines.append(line.line('  This is a test of ending whitespace'))
        oFile.lines.append(line.line('  This is a test of ending whitespace '))
        oFile.lines.append(line.line('  This is a test of ending whitespace'))
        oFile.lines.append(line.line('  This is a test of ending whitespace  '))
        oFile.lines.append(line.line('  This is a test of ending whitespace'))
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_002(self):
        oRule = whitespace.rule_002()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'whitespace')
        self.assertEqual(oRule.identifier, '002')

        oFile = vhdlFile.vhdlFile(sFileName)

        dExpected = []
        oFile.lines.append(line.line('  This is a test of tabs\t'))
        oFile.lines.append(line.line('\tThis is a test of tabs'))
        oFile.lines.append(line.line('  This is a test of tabs'))
        oFile.lines.append(line.line('  This is a test of tabs'))
        oFile.lines.append(line.line('  This is a \t test of tabs'))
        oFile.lines.append(line.line('  This is a test of tabs'))
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_003(self):
        oRule = whitespace.rule_003()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'whitespace')
        self.assertEqual(oRule.identifier, '003')

        oFile = vhdlFile.vhdlFile(sFileName)

        dExpected = []
        oFile.lines.append(line.line('  This is a test of tabs;'))
        oFile.lines.append(line.line('  This is a test of tabs ;'))
        oFile.lines.append(line.line('  This is a test of tabs;'))
        oFile.lines.append(line.line('  This is a test of tabs    ;'))
        oFile.lines.append(line.line('  This is a test; of tabs'))
        oFile.lines.append(line.line('  This is a test ; of tabs'))
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_004(self):
        oRule = whitespace.rule_004()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'whitespace')
        self.assertEqual(oRule.identifier, '004')

        oFile = vhdlFile.vhdlFile(sFileName)

        dExpected = []
        oFile.lines.append(line.line('  This is a test of commas,'))
        oFile.lines.append(line.line('  This is a test of commas ,'))
        oFile.lines.append(line.line('  This is a test of commas,'))
        oFile.lines.append(line.line('  This is a test of commas    ,'))
        oFile.lines.append(line.line('  This is a test, of commas'))
        oFile.lines.append(line.line('  This is a test , of commas'))
        oFile.lines.append(line.line('  This is a test, of commas -- This is a comment ,'))
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)
        self.assertEqual(oFile.lines[7].line, '  This is a test, of commas -- This is a comment ,')

    def test_fix_007(self):
        oRule = whitespace.rule_007()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'whitespace')
        self.assertEqual(oRule.identifier, '007')

        oFile = vhdlFile.vhdlFile(sFileName)

        dExpected = []
        oFile.lines.append(line.line('  This is a test,of commas (failure )'))
        oFile.lines.append(line.line('  This is a test, of commas (pass)'))
        oFile.lines.append(line.line('  This is a test of commas,(failure   )'))
        oFile.lines.append(line.line('  This is a test of commas -- 1,2,3,4 (PASS)'))
        oFile.lines.append(line.line('   ) pass'))
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_008(self):
        oRule = whitespace.rule_008()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'whitespace')
        self.assertEqual(oRule.identifier, '008')

        oFile = vhdlFile.vhdlFile(sFileName)

        dExpected = []
        oFile.lines.append(line.line('A  std_logic_vector (7 downto 0)'))
        oFile.lines.append(line.line('  std_logic_vector(7 downto 0)'))
        oFile.lines.append(line.line('  std_logic_vector   (7 downto 0)'))
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)
        self.assertEqual(oFile.lines[1].line, 'A  std_logic_vector(7 downto 0)')
        self.assertEqual(oFile.lines[2].line, '  std_logic_vector(7 downto 0)')
        self.assertEqual(oFile.lines[3].line, '  std_logic_vector(7 downto 0)')




from vsg import rule
from vsg import fix

import re


class rule_006(rule.rule):
    '''
    Entity rule 006 checks the "is" keyword is lower case.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'entity'
        self.identifier = '006'
        self.solution = 'Change "is" keyword to lowercase.'
        self.phase = 6

    def analyze(self, oFile):
        for iLineNumber, oLine in enumerate(oFile.lines):
            if oLine.isEntityDeclaration:
                lLine = oLine.line.split()
                if len(lLine) > 2:
                    if not re.match('^\s*\S+\s+\S+\s\s*is', oLine.line):
                        self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            fix.lower_case(self, oFile.lines[iLineNumber], 'is')




from vsg import rule
from vsg import check
from vsg import fix


class keyword_alignment_rule(rule.rule):
    '''
    Checks for keyword alignment in consecutive concurrent statements.

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.

    Attributes
    ----------

    sKeyword : string
       The keyword to align.

    sStartGroupTrigger: string
       The line attribute which marks the start of a group to align.

    sEndGroupTrigger: string
       The line attribute which ends the group to align.
       NOTE:  This is inversed to allow for grouping of consecutive lines.
    '''

    def __init__(self, name, identifier):
        rule.rule.__init__(self, name, identifier)
        self.phase = 5
        self.sKeyword = None
        self.sStartGroupTrigger = None
        self.sEndGroupTrigger = None

    def analyze(self, oFile):
        lGroup = []
        fGroupFound = False
        iStartGroupIndex = None
        for iLineNumber, oLine in enumerate(oFile.lines):
            if oLine.__dict__[self.sStartGroupTrigger] and not fGroupFound:
                fGroupFound = True
                iStartGroupIndex = iLineNumber
            if not oLine.__dict__[self.sEndGroupTrigger] and fGroupFound:
                fGroupFound = False
                check.keyword_alignment(self, iStartGroupIndex, self.sKeyword, lGroup)
                lGroup = []
                iStartGroupIndex = None
            if fGroupFound:
                lGroup.append(oLine)

    def _fix_violations(self, oFile):
        fix.keyword_alignment(self, oFile)



import os
import unittest

from vsg.rules import constant
from vsg import vhdlFile


# Read in test file used for all tests
oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','constant','constant_test_input.vhd'))

class testFixRuleConstantMethods(unittest.TestCase):

    def test_fix_rule_001(self):
        oRule = constant.rule_001()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_002(self):
        oRule = constant.rule_002()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_003(self):
        oRule = constant.rule_003()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_004(self):
        oRule = constant.rule_004()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_005(self):
        oRule = constant.rule_005()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_006(self):
        oRule = constant.rule_006()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_007(self):
        oRule = constant.rule_007()
        self.assertFalse(oRule.fixable)

    def test_fix_rule_009(self):
        oRule = constant.rule_009()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_010(self):
        oRule = constant.rule_010()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_011(self):
        oRule = constant.rule_011()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)
        self.assertEqual(oFile.lines[9].line, '  constant const   : std_logic :=\'0\';')

    def test_fix_rule_012(self):
        oRule = constant.rule_012()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)
        self.assertEqual(oFile.lines[22].line, '  (')
        self.assertEqual(oFile.lines[23].line, '    0,')
        self.assertEqual(oFile.lines[24].line, '    1,')
        self.assertEqual(oFile.lines[25].line, '    2,')
        self.assertEqual(oFile.lines[26].line, '    3')
        self.assertEqual(oFile.lines[27].line, '  );')



import os
import unittest


from vsg import vhdlFile
from vsg import rule_list
from vsg.tests import utils

oIteration = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'iteration_synth.vhd'))


class testCodeExample(unittest.TestCase):

    def test_iteration_synth(self):
        oRuleList = rule_list.rule_list(oIteration)
        oRuleList.fix(7)
        lExpected = ['']
        utils.read_file(os.path.join(os.path.dirname(__file__),'iteration_synth.fixed.vhd'), lExpected)
        for iLineNumber, sLine in enumerate(lExpected):
            self.assertEqual(oIteration.lines[iLineNumber].line, sLine)



import os
import unittest

from vsg.rules import constant
from vsg import vhdlFile


# Read in test file used for all tests
oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','constant','constant_test_input.vhd'))

class testRuleConstantMethods(unittest.TestCase):

    def test_rule_001(self):
        oRule = constant.rule_001()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'constant')
        self.assertEqual(oRule.identifier, '001')
        dExpected = [8,9,10]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_002(self):
        oRule = constant.rule_002()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'constant')
        self.assertEqual(oRule.identifier, '002')
        dExpected = [7,8]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_003(self):
        oRule = constant.rule_003()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'constant')
        self.assertEqual(oRule.identifier, '003')
        dExpected = [7]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_004(self):
        oRule = constant.rule_004()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'constant')
        self.assertEqual(oRule.identifier, '004')
        dExpected = [8]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_005(self):
        oRule = constant.rule_005()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'constant')
        self.assertEqual(oRule.identifier, '005')
        dExpected = [8,9]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_006(self):
        oRule = constant.rule_006()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'constant')
        self.assertEqual(oRule.identifier, '006')
        dExpected = [10]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_007(self):
        oRule = constant.rule_007()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'constant')
        self.assertEqual(oRule.identifier, '007')
        dExpected = [10]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_009(self):
        oRule = constant.rule_009()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'constant')
        self.assertEqual(oRule.identifier, '009')
        dExpected = ['3-15']
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_010(self):
        oRule = constant.rule_010()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'constant')
        self.assertEqual(oRule.identifier, '010')
        dExpected = [9]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_011(self):
        oRule = constant.rule_011()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'constant')
        self.assertEqual(oRule.identifier, '011')
        dExpected = [9]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_012(self):
        oRule = constant.rule_012()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'constant')
        self.assertEqual(oRule.identifier, '012')
        dExpected = [11,22,23,25,27]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)



import os

import unittest

from vsg.rules import file_statement
from vsg import vhdlFile


# Read in test file used for all tests
oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'file_test_input.vhd'))


class testFixRuleFileMethods(unittest.TestCase):

    def test_fix_rule_001(self):
        oRule = file_statement.rule_001()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)
        self.assertEqual(oFile.lines[9].line, '  FILE     defaultImage : load_file_type open read_mode is load_file_name;')
        self.assertEqual(oFile.lines[11].line, '  file defaultImage : load_file_type open read_mode')
        self.assertEqual(oFile.lines[12].line, '    is load_file_name;')

    def test_fix_rule_002(self):
        oRule = file_statement.rule_002()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)
        self.assertEqual(oFile.lines[9].line, '  file     defaultImage : load_file_type open read_mode is load_file_name;')

    def test_fix_rule_003(self):
        oRule = file_statement.rule_003()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)
        self.assertEqual(oFile.lines[9].line, '  file defaultImage : load_file_type open read_mode is load_file_name;')




import os

import unittest
import sys
sys.path.append('vsg')

from vsg.rules import variable
from vsg import vhdlFile

# Read in test file used for all tests
oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','variable','variable_test_input.vhd'))

class testFixRuleVariableMethods(unittest.TestCase):

    def test_fix_rule_001(self):
        oRule = variable.rule_001()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_002(self):
        oRule = variable.rule_002()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_003(self):
        oRule = variable.rule_003()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_004(self):
        oRule = variable.rule_004()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_005(self):
        oRule = variable.rule_005()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_006(self):
        oRule = variable.rule_006()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_007(self):
        oRule = variable.rule_007()
        self.assertFalse(oRule.fixable)

    def test_fix_rule_009(self):
        oRule = variable.rule_009()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_010(self):
        oRule = variable.rule_010()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)



import os

import unittest
from vsg import vhdlFile

oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','procedure','procedure_test_input.vhd'))

class testVhdlFileProcedureAssignments(unittest.TestCase):


    def test_isProcedureKeyword(self):
        lExpected = [4,6,14,16,26,30,40,44,54,63,72,84]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isProcedureKeyword:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isProcedureBegin(self):
        lExpected = [27,35,41,49,60,69,79,91]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isProcedureBegin:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isProcedureEnd(self):
        lExpected = [4,10,14,20,28,36,42,50,61,70,80,92]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isProcedureEnd:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_insideProcedure(self):
        lExpected = [4,6,7,8,9,10,14,16,17,18,19,20,26,27,28,30,31,32,33,34,35,36,40,41,42,44,45,46,47,48,49,50,54,55,56,57,58,59,60,61,63,64,65,66,67,68,69,70,72,73,74,75,76,77,78,79,80,84,85,86,87,88,89,90,91,92]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.insideProcedure:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_insideProcedureDeclarative(self):
        lExpected = [89,90]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.insideProcedureDeclarative:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_ProcedureIndent(self):
        #           [   0,   1,2,   3,4,   5,6,7,8,9,10,  11]
        lExpected = [None,None,0,None,1,None,1,2,2,2, 2,None]
        # Generic actual list
        lActual = []
        iMaxCheck = len(lExpected)
        for iIndex, oLine in enumerate(oFile.lines):
            if iIndex == iMaxCheck:
                break
            lActual.append(oLine.indentLevel)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isProcedureParameter(self):
        lExpected = [7,8,9,10,17,18,19,20,31,32,33,34,44,45,46,47,48,55,56,57,58,64,65,66,67,73,74,75,76,85,86,87,88]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isProcedureParameter:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isProcedureParameterEnd(self):
        lExpected = [10,20,34,48,58,68,77,88]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isProcedureParameterEnd:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isProcedureIs(self):
        lExpected = [26,34,40,48,59,68,78,88]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isProcedureIs:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_is_not_constant(self):
        lExpected = []
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isConstant:
                lActual.append(iIndex)  # pragma: no cover
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_is_not_signal(self):
        lExpected = []
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isSignal:
                lActual.append(iIndex)  # pragma: no cover
        # Compare
        self.assertEqual(lActual, lExpected)




from vsg import rule

import re


class rule_007(rule.rule):
    '''
    Port rule 007 checks for four spaces after the "in" keyword in a port declaration for "in" ports.
    '''

    def __init__(self):
        rule.rule.__init__(self, 'port', '007')
        self.solution = 'Change the number of spaces after the "in" keyword to four spaces.'
        self.phase = 2

    def analyze(self, oFile):
        for iLineNumber, oLine in enumerate(oFile.lines):
            if oLine.isPortDeclaration and re.match('^\s*\S+\s*:\s*in\s', oLine.lineLower):
                if not re.match('^\s*\S+\s*:\s*in\s\s\s\s\S+', oLine.lineLower):
                    self.add_violation(iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            oLine = oFile.lines[iLineNumber]
            oLine.update_line(re.sub(r'^(\s*\S+\s*:\s*in)(\s*)', r'\1    ', oLine.line, flags=re.IGNORECASE))



import os

import unittest

from vsg.rules import instantiation
from vsg import vhdlFile


oFilePort = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'instantiation_test_input.vhd'))
oFileGeneric = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'instantiation_generic_test_input.vhd'))
oFileComment = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'instantiation_comment_test_input.vhd'))
oFilePositional = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'instantiation_positional_test_input.vhd'))
oFileDirect = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'instantiation_direct_test_input.vhd'))

class testRuleInstantiationMethods(unittest.TestCase):

    def test_rule_001(self):
        oRule = instantiation.rule_001()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'instantiation')
        self.assertEqual(oRule.identifier, '001')

        dExpected = [19,21,25,26,27,31,32,33,34,35,37,42,44,52]
        oRule.analyze(oFilePort)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_002(self):
        oRule = instantiation.rule_002()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'instantiation')
        self.assertEqual(oRule.identifier, '002')

        dExpected = [44,52]
        oRule.analyze(oFilePort)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_003(self):
        oRule = instantiation.rule_003()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'instantiation')
        self.assertEqual(oRule.identifier, '003')

        dExpected = [36,52]
        oRule.analyze(oFilePort)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_004(self):
        oRule = instantiation.rule_004()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'instantiation')
        self.assertEqual(oRule.identifier, '004')

        dExpected = [23,29,36]
        oRule.analyze(oFilePort)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_005(self):
        oRule = instantiation.rule_005()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'instantiation')
        self.assertEqual(oRule.identifier, '005')

        dExpected = [52]
        oRule.analyze(oFilePort)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_006(self):
        oRule = instantiation.rule_006()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'instantiation')
        self.assertEqual(oRule.identifier, '006')

        dExpected = [31,37,52]
        oRule.analyze(oFilePort)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_007(self):
        oRule = instantiation.rule_007()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'instantiation')
        self.assertEqual(oRule.identifier, '007')

        dExpected = [55]
        oRule.analyze(oFilePort)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_008(self):
        oRule = instantiation.rule_008()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'instantiation')
        self.assertEqual(oRule.identifier, '008')

        dExpected = [17,52]
        oRule.analyze(oFilePort)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_009(self):
        oRule = instantiation.rule_009()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'instantiation')
        self.assertEqual(oRule.identifier, '009')

        dExpected = [23,52]
        oRule.analyze(oFilePort)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_010(self):
        oRule = instantiation.rule_010()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'instantiation')
        self.assertEqual(oRule.identifier, '010')

        dExpected = ['18-22']
        oRule.analyze(oFilePort)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_011(self):
        oRule = instantiation.rule_011()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'instantiation')
        self.assertEqual(oRule.identifier, '011')

        dExpected = [19,20,21,25,26,27,32,33,34,39,41,46,47,48,53,54,55,71,72]
        oRule.analyze(oFilePort)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_012(self):
        oRule = instantiation.rule_012()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'instantiation')
        self.assertEqual(oRule.identifier, '012')

        dExpected = [63]
        oRule.analyze(oFileGeneric)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_013(self):
        oRule = instantiation.rule_013()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'instantiation')
        self.assertEqual(oRule.identifier, '013')

        dExpected = [22,33]
        oRule.analyze(oFileGeneric)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_014(self):
        oRule = instantiation.rule_014()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'instantiation')
        self.assertEqual(oRule.identifier, '014')

        dExpected = [46,65,73]
        oRule.analyze(oFileGeneric)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_015(self):
        oRule = instantiation.rule_015()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'instantiation')
        self.assertEqual(oRule.identifier, '015')

        dExpected = ['44-46','54-56']
        oRule.analyze(oFileGeneric)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_016(self):
        oRule = instantiation.rule_016()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'instantiation')
        self.assertEqual(oRule.identifier, '016')

        dExpected = [34,46,55]
        oRule.analyze(oFileGeneric)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_017(self):
        oRule = instantiation.rule_017()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'instantiation')
        self.assertEqual(oRule.identifier, '017')

        dExpected = [54]
        oRule.analyze(oFileGeneric)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_001_generics(self):
        oRule = instantiation.rule_001()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'instantiation')
        self.assertEqual(oRule.identifier, '001')

        dExpected = [22,33,45,46,73]
        oRule.analyze(oFileGeneric)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_018(self):
        oRule = instantiation.rule_018()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'instantiation')
        self.assertEqual(oRule.identifier, '018')

        dExpected = [26,33,54,66]
        oRule.analyze(oFileGeneric)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_019(self):
        oRule = instantiation.rule_019()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'instantiation')
        self.assertEqual(oRule.identifier, '019')

        dExpected = [22,28,35,66]
        oRule.analyze(oFilePort)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_020(self):
        oRule = instantiation.rule_020()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'instantiation')
        self.assertEqual(oRule.identifier, '020')

        dExpected = [58]
        oRule.analyze(oFilePort)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_021(self):
        oRule = instantiation.rule_021()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'instantiation')
        self.assertEqual(oRule.identifier, '021')

        dExpected = [65,77]
        oRule.analyze(oFilePort)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_022(self):
        oRule = instantiation.rule_022()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'instantiation')
        self.assertEqual(oRule.identifier, '022')

        dExpected = [26,27]
        oRule.analyze(oFilePort)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_023(self):
        oRule = instantiation.rule_023()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'instantiation')
        self.assertEqual(oRule.identifier, '023')

        dExpected = [24,29,31]
        oRule.analyze(oFileComment)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_024(self):
        oRule = instantiation.rule_024()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'instantiation')
        self.assertEqual(oRule.identifier, '024')

        dExpected = [7,9,10,13,14]
        oRule.analyze(oFilePositional)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_025(self):
        oRule = instantiation.rule_025()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'instantiation')
        self.assertEqual(oRule.identifier, '025')

        dExpected = [81]
        oRule.analyze(oFileGeneric)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_026(self):
        oRule = instantiation.rule_026()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'instantiation')
        self.assertEqual(oRule.identifier, '026')

        dExpected = [86]
        oRule.analyze(oFileGeneric)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_027(self):
        oRule = instantiation.rule_027()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'instantiation')
        self.assertEqual(oRule.identifier, '027')

        dExpected = [13,20]
        oRule.analyze(oFileDirect)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_028(self):
        oRule = instantiation.rule_028()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'instantiation')
        self.assertEqual(oRule.identifier, '028')

        dExpected = [20]
        oRule.analyze(oFileDirect)
        self.assertEqual(oRule.violations, dExpected)



import os

import unittest
from vsg import vhdlFile

oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','sequential','sequential_test_input.vhd'))

class testVhdlFileSequentialAssignments(unittest.TestCase):


    def test_isSequential_assignment(self):
        lExpected = [13,14,15,20,21,22,26,27,28,33,34,38,39,40,53,56,57,63,65,66,71,73,75,80,81,83,88,89,90,92,99]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isSequential:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_insideSequential_assignment(self):
        lExpected = [13,14,15,20,21,22,26,27,28,33,34,38,39,40,53,54,55,56,57,58,63,65,66,71,73,74,75,80,81,83,88,89,90,92,93,94,95,99,100]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.insideSequential:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isSequentialEnd_assignment(self):
        lExpected = [13,14,15,20,21,22,26,27,28,33,34,38,39,40,55,56,58,63,65,66,71,74,75,80,81,83,88,89,90,95,100]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isSequentialEnd:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_sequentialAlignmentColumn_assignment(self):
        #lExpected = [13,14,15,20,21,22,26,27,28,33,34,38,39,40,53,56,57,63,65,66,71,73,75,80,81,83,88,89,90]
        lExpected =  [ 6, 4, 6,11,10,10,10,10, 8,10,10, 9,10, 9, 8, 7, 8,10,12,13,10,15,14,10, 8,12,10, 8,11,10, 6]
        # Generic actual list
        lActual = []
        for oLine in oFile.lines:
            if oLine.isSequential:
                lActual.append(oLine.sequentialAlignmentColumn)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_sequentialIndentLevel_assignment(self):
        #lExpected = [13,14,15,20,21,22,26,27,28,33,34,38,39,40,53,56,57,63,65,66,71,73,75,80,81,83,88,89,90]
        lExpected =  [ 2, 2, 2, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 2, 2, 2, 4, 5, 5, 4, 5, 5, 4, 4, 5, 4, 4, 4, 4, 2]
        # Generic actual list
        lActual = []
        for oLine in oFile.lines:
            if oLine.isSequential:
                lActual.append(oLine.indentLevel)
        # Compare
        self.assertEqual(lActual, lExpected)



import os
import unittest

from vsg.rules import concurrent
from vsg import vhdlFile

# Read in test file used for all tests
oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','concurrent','concurrent_test_input.vhd'))

class testRuleConcurrentMethods(unittest.TestCase):

    def test_rule_001(self):
        oRule = concurrent.rule_001()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'concurrent')
        self.assertEqual(oRule.identifier, '001')
        dExpected = [7,11,24,32,33]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_002(self):
        oRule = concurrent.rule_002()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'concurrent')
        self.assertEqual(oRule.identifier, '002')
        dExpected = [7,8,24,32,33]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_003(self):
        oRule = concurrent.rule_003()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'concurrent')
        self.assertEqual(oRule.identifier, '003')
        dExpected = [28,29,30]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_004(self):
        oRule = concurrent.rule_004()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'concurrent')
        self.assertEqual(oRule.identifier, '004')
        dExpected = [7,8,32,33]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_005(self):
        oRule = concurrent.rule_005()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'concurrent')
        self.assertEqual(oRule.identifier, '005')
        dExpected = [32,33,34,35]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_006(self):
        oRule = concurrent.rule_006()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'concurrent')
        self.assertEqual(oRule.identifier, '006')
        dExpected = ['6-11','23-24','32-36']
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_007(self):
        oRule = concurrent.rule_007()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'concurrent')
        self.assertEqual(oRule.identifier, '007')
        dExpected = [44,48]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_008(self):
        oRule = concurrent.rule_008()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'concurrent')
        self.assertEqual(oRule.identifier, '008')
        dExpected = ['6-11']
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)



import os

import unittest

from vsg.rules import generic
from vsg import vhdlFile


oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'generic_test_input.vhd'))
oFileMultiple = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'generic_multiple_on_one_line_test_input.vhd'))


class testFixRuleGenericMethods(unittest.TestCase):


    def test_fix_rule_001(self):
        oRule = generic.rule_001()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])
        self.assertFalse(oFile.lines[81].isBlank)

    def test_fix_rule_002(self):
        oRule = generic.rule_002()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_003(self):
        oRule = generic.rule_003()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_004(self):
        oRule = generic.rule_004()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_005(self):
        oRule = generic.rule_005()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_006(self):
        oRule = generic.rule_006()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])
        self.assertEqual(oFile.lines[37].line,'    G_GeneRIC2 : std_logic := \'1\'')
        self.assertEqual(oFile.lines[52].line,'    G_GENERIC1 : STD_LOGIC := \'0\';')
        self.assertEqual(oFile.lines[53].line,'    G_GENERIC2 : std_logic := \'1\'')

    def test_fix_rule_007(self):
        oRule = generic.rule_007()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_008(self):
        oRule = generic.rule_008()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_009(self):
        oRule = generic.rule_009()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_010(self):
        oRule = generic.rule_010()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])
        self.assertEqual(oFile.lines[96].indentLevel, oFile.lines[95].indentLevel)
        self.assertEqual(oFile.lines[97].indentLevel, oFile.lines[96].indentLevel - 1)

#    def test_fix_rule_011(self):
#        oRule = generic.rule_011()
#        oRule.fix(oFile)
#        oRule.analyze(oFile)
#        self.assertEqual(oRule.violations, [])

    def test_fix_rule_012(self):
        oRule = generic.rule_012()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_013(self):
        oRule = generic.rule_013()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])
        self.assertEqual(oFile.lines[140].indentLevel, oFile.lines[139].indentLevel + 1)

    def test_fix_rule_014(self):
        oRule = generic.rule_014()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_015(self):
        oRule = generic.rule_015()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_016(self):
        oRule = generic.rule_016()
        oRule.fix(oFileMultiple)
        oRule.analyze(oFileMultiple)
        self.assertEqual(oRule.violations, [])
        self.assertEqual(oFileMultiple.lines[4].line,'  generic (')
        self.assertEqual(oFileMultiple.lines[5].line,'    G_GENERIC1 : std_logic := \'0\';')
        self.assertEqual(oFileMultiple.lines[6].line,'G_GENERIC2 : std_logic := \'1\';')
        self.assertEqual(oFileMultiple.lines[7].line,'G_GENERIC3 : std_logic := \'1\';')
        self.assertEqual(oFileMultiple.lines[8].line,'G_GENERIC4 : std_logic := \'1\';')
        self.assertEqual(oFileMultiple.lines[9].line,'G_GENERIC5 : std_logic := \'1\'')
        self.assertEqual(oFileMultiple.lines[10].line,'  );')



import os

import unittest

from vsg.rules import if_statement
from vsg import vhdlFile
from vsg import rule_list

# Read in test file used for all tests
oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','if_statement','if_test_input.vhd'))
oFileCase = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','if_statement','if_case_test_input.vhd'))
oFileCompress = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'if_compressed_line_test_input.vhd'))


class testFixRuleIfMethods(unittest.TestCase):

    def test_fix_rule_001(self):
        oRule = if_statement.rule_001()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_002(self):
        oRule = if_statement.rule_002()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oFile.lines[13].line, '    if ( a = 1 or d = 20 or    -- this if should not be replaced')
        self.assertEqual(oFile.lines[14].line, '       g = 34 or x = 3000 ) then')
        self.assertEqual(oFile.lines[24].line, '   elsif ( z = 45 and f = 45 ) then')
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_003(self):
        oRule = if_statement.rule_003()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_004(self):
        oRule = if_statement.rule_004()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_005(self):
        oRule = if_statement.rule_005()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_006(self):
        oRule = if_statement.rule_006()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_006_case(self):
        oFileCase = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'if_case_test_input.vhd'))
        oRule = if_statement.rule_006()
        dExpected = []
        oRule.fix(oFileCase)
        oRule.analyze(oFileCase)
        self.assertEqual(oRule.violations, dExpected)
        self.assertTrue(oFileCase.lines[10].isBlank)
        self.assertTrue(oFileCase.lines[20].isBlank)

    def test_fix_rule_007(self):
        oRule = if_statement.rule_007()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_007_case(self):
        oFileCase = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'if_case_test_input.vhd'))
        oRule = if_statement.rule_007()
        dExpected = []
        oRule.fix(oFileCase)
        oRule.analyze(oFileCase)
        self.assertEqual(oRule.violations, dExpected)
        self.assertTrue(oFileCase.lines[18].isBlank)

    def test_fix_rule_008(self):
        oRule = if_statement.rule_008()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_008_case(self):
        oFileCase = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'if_case_test_input.vhd'))
        oRule = if_statement.rule_008()
        dExpected = []
        oRule.fix(oFileCase)
        oRule.analyze(oFileCase)
        self.assertEqual(oRule.violations, dExpected)
        self.assertTrue(oFileCase.lines[40].isBlank)

    def test_fix_rule_009(self):
        oRule = if_statement.rule_009()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_010(self):
        oRule = if_statement.rule_010()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_010_case(self):
        oFileCase = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'if_case_test_input.vhd'))
        oRule = if_statement.rule_010()
        dExpected = []
        oRule.fix(oFileCase)
        oRule.analyze(oFileCase)
        self.assertEqual(oRule.violations, dExpected)
        self.assertTrue(oFileCase.lines[29].isBlank)

    def test_fix_rule_011(self):
        oRule = if_statement.rule_011()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_011_case(self):
        oFileCase = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'if_case_test_input.vhd'))
        oRule = if_statement.rule_011()
        dExpected = []
        oRule.fix(oFileCase)
        oRule.analyze(oFileCase)
        self.assertEqual(oRule.violations, dExpected)
        self.assertTrue(oFileCase.lines[32].isBlank)

    def test_fix_rule_012(self):
        oRule = if_statement.rule_012()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_013(self):
        oRule = if_statement.rule_013()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_014(self):
        oRule = if_statement.rule_014()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_024(self):
        oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','if_statement','if_test_input.vhd'))
        oRule = if_statement.rule_024()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)
        self.assertEqual(oFile.lines[107].line, '    if (a = 2) then')
        self.assertEqual(oFile.lines[108].line, ' b <= \'1\'; else b <= \'0\'; end if;')
        self.assertEqual(oFile.lines[108].indentLevel, oFile.lines[107].indentLevel + 1)
        self.assertFalse(oFile.lines[107].isEndIfKeyword)
        self.assertFalse(oFile.lines[107].isElseKeyword)

    def test_fix_rule_021(self):
        oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','if_statement','if_test_input.vhd'))
        oRule = if_statement.rule_021()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)
        self.assertEqual(oFile.lines[105].line, '    if (a = 2) then b <= \'1\'; ')
        self.assertEqual(oFile.lines[106].line, 'else b <= \'0\'; end if;')
        self.assertEqual(oFile.lines[106].indentLevel, oFile.lines[105].indentLevel)


    def test_fix_rule_020(self):
        oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','if_statement','if_test_input.vhd'))
        oRule = if_statement.rule_020()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_015(self):
        oRule = if_statement.rule_015()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_022(self):
        oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','if_statement','if_test_input.vhd'))
        oRule = if_statement.rule_022()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)
        self.assertEqual(oFile.lines[102].line, '    else')
        self.assertEqual(oFile.lines[103].line, ' g <= x;')
        self.assertEqual(oFile.lines[103].indentLevel, oFile.lines[102].indentLevel + 1)

    def test_fix_compressed_line(self):
        oRuleList = rule_list.rule_list(oFileCompress)
        oRuleList.fix(7)
#        utils.debug_lines(oFileCompress, 9, 10)
        self.assertEqual(oFileCompress.lines[10].line, '    if (A = \'1\' and B = \'1\') then')
        self.assertEqual(oFileCompress.lines[11].line, '      X <= \'1\';')
        self.assertFalse(oFileCompress.lines[11].isVariableAssignment)
        self.assertTrue(oFileCompress.lines[11].isSequential)
        self.assertEqual(oFileCompress.lines[12].line, '    elsif (C = \'0\') then')
        self.assertEqual(oFileCompress.lines[13].line, '      Y <= \'0\';')
        self.assertFalse(oFileCompress.lines[13].isVariableAssignment)
        self.assertTrue(oFileCompress.lines[13].isSequential)
        self.assertEqual(oFileCompress.lines[14].line, '    else')
        self.assertEqual(oFileCompress.lines[15].line, '      W := \'0\';')
        self.assertTrue(oFileCompress.lines[15].isVariableAssignment)
        self.assertFalse(oFileCompress.lines[15].isSequential)

        self.assertEqual(oFileCompress.lines[18].line, '    if (A = \'1\' and B = \'1\') then')
        self.assertEqual(oFileCompress.lines[19].line, '      X <= \'1\';')
        self.assertEqual(oFileCompress.lines[20].line, '    elsif (C = \'0\') then')
        self.assertEqual(oFileCompress.lines[21].line, '      Y <= \'0\';')
        self.assertEqual(oFileCompress.lines[22].line, '    else')
        self.assertEqual(oFileCompress.lines[23].line, '      W <= \'0\';')

    def test_fix_rule_025(self):
        oRule = if_statement.rule_025()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)
        self.assertEqual(oFile.lines[13].line, '    if ( a = 1 or d = 20 or    -- this if should not be replaced')

    def test_fix_rule_026(self):
        oRule = if_statement.rule_026()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)
        self.assertEqual(oFile.lines[24].line, '    elsif ( z = 45 and f = 45 ) then')

    def test_fix_rule_027(self):
        oRule = if_statement.rule_027()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)
        self.assertEqual(oFile.lines[86].line, '    else')

    def test_fix_rule_028(self):
        oRule = if_statement.rule_028()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)
        self.assertEqual(oFile.lines[17].line, '    end if;')
        self.assertEqual(oFile.lines[27].line, '    end if;')

    def test_fix_rule_029(self):
        oRule = if_statement.rule_029()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)
        self.assertEqual(oFile.lines[14].line, '        g = 34 or x = 3000 ) then')



import os

import unittest
from vsg import vhdlFile

oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','with_statement','with_test_input.vhd'))

class testVhdlFileMethods(unittest.TestCase):

    def test_isWithKeyword_assignment(self):
        lExpected = [6]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isWithKeyword:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)



import os

import unittest

from vsg.rules import attribute
from vsg import vhdlFile


# Read in test file used for all tests
oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'attribute_test_input.vhd'))


class testFixRuleAttributeMethods(unittest.TestCase):

    def test_fix_rule_001(self):
        oRule = attribute.rule_001()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)
        self.assertEqual(oFile.lines[9].line, '  ATTRIBUTE ram_init_file : string;')
        self.assertEqual(oFile.lines[11].line, '  attribute      ram_init_file of ram_block :')
        self.assertEqual(oFile.lines[12].line, '    signal is "contents.mif";')

    def test_fix_rule_002(self):
        oRule = attribute.rule_002()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)
        self.assertEqual(oFile.lines[9].line, '  attribute ram_init_file : string;')

    def test_fix_rule_003(self):
        oRule = attribute.rule_003()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)
        self.assertEqual(oFile.lines[11].line, '  attribute ram_init_file of ram_block :')




import os

import unittest

from vsg.rules import if_statement
from vsg import vhdlFile

# Read in test file used for all tests
oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','if_statement','if_test_input.vhd'))
oFileCase = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','if_statement','if_case_test_input.vhd'))


class testRuleIfMethods(unittest.TestCase):

    def test_rule_001(self):
        oRule = if_statement.rule_001()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'if')
        self.assertEqual(oRule.identifier, '001')
        dExpected = [13,33]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_002(self):
        oRule = if_statement.rule_002()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'if')
        self.assertEqual(oRule.identifier, '002')
        dExpected = [8,13,24,33,41,46,52]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_003(self):
        oRule = if_statement.rule_003()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'if')
        self.assertEqual(oRule.identifier, '003')
        dExpected = [57]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_004(self):
        oRule = if_statement.rule_004()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'if')
        self.assertEqual(oRule.identifier, '004')
        dExpected = [32,57,73]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_005(self):
        oRule = if_statement.rule_005()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'if')
        self.assertEqual(oRule.identifier, '005')
        dExpected = [73]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_006(self):
        oRule = if_statement.rule_006()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'if')
        self.assertEqual(oRule.identifier, '006')
        dExpected = [68,73,80]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_006_case(self):
        oRule = if_statement.rule_006()
        dExpected = [9,20]
        oRule.analyze(oFileCase)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_007(self):
        oRule = if_statement.rule_007()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'if')
        self.assertEqual(oRule.identifier, '007')
        dExpected = [73]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_007_case(self):
        oRule = if_statement.rule_007()
        dExpected = [20]
        oRule.analyze(oFileCase)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_008(self):
        oRule = if_statement.rule_008()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'if')
        self.assertEqual(oRule.identifier, '008')
        dExpected = [78,89]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_008_case(self):
        oRule = if_statement.rule_008()
        dExpected = [42]
        oRule.analyze(oFileCase)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_009(self):
        oRule = if_statement.rule_009()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'if')
        self.assertEqual(oRule.identifier, '009')
        dExpected = [20,21,67,68,115]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_010(self):
        oRule = if_statement.rule_010()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'if')
        self.assertEqual(oRule.identifier, '010')
        dExpected = [85]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_010_case(self):
        oRule = if_statement.rule_010()
        dExpected = [31]
        oRule.analyze(oFileCase)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_011(self):
        oRule = if_statement.rule_011()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'if')
        self.assertEqual(oRule.identifier, '011')
        dExpected = [85]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_011_case(self):
        oRule = if_statement.rule_011()
        dExpected = [31]
        oRule.analyze(oFileCase)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_012(self):
        oRule = if_statement.rule_012()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'if')
        self.assertEqual(oRule.identifier, '012')
        dExpected = [24,108]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_013(self):
        oRule = if_statement.rule_013()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'if')
        self.assertEqual(oRule.identifier, '013')
        dExpected = [85,109]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_014(self):
        oRule = if_statement.rule_014()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'if')
        self.assertEqual(oRule.identifier, '014')
        dExpected = [11,27,36,39,60,110]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_015(self):
        oRule = if_statement.rule_015()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'if')
        self.assertEqual(oRule.identifier, '015')
        dExpected = [117]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_020(self):
        oRule = if_statement.rule_020()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'if')
        self.assertEqual(oRule.identifier, '020')
        dExpected = [105,110]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_021(self):
        oRule = if_statement.rule_021()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'if')
        self.assertEqual(oRule.identifier, '021')
        dExpected = [105,109]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_022(self):
        oRule = if_statement.rule_022()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'if')
        self.assertEqual(oRule.identifier, '022')
        dExpected = [102,105]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_023(self):
        oRule = if_statement.rule_023()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'if')
        self.assertEqual(oRule.identifier, '023')
        dExpected = [108]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_024(self):
        oRule = if_statement.rule_024()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'if')
        self.assertEqual(oRule.identifier, '024')
        dExpected = [98,99,105,109]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_025(self):
        oRule = if_statement.rule_025()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'if')
        self.assertEqual(oRule.identifier, '025')
        dExpected = [13]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_026(self):
        oRule = if_statement.rule_026()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'if')
        self.assertEqual(oRule.identifier, '026')
        dExpected = [24]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_027(self):
        oRule = if_statement.rule_027()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'if')
        self.assertEqual(oRule.identifier, '027')
        dExpected = [94]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_028(self):
        oRule = if_statement.rule_028()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'if')
        self.assertEqual(oRule.identifier, '028')
        dExpected = [17,27]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_029(self):
        oRule = if_statement.rule_029()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'if')
        self.assertEqual(oRule.identifier, '029')
        dExpected = [14]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)



import re


def process(dVars, oLine):
    classify_process_keyword(dVars, oLine)
    if oLine.insideProcess:
        classify_process_sensitivity_list(dVars, oLine)
        classify_process_begin_keyword(dVars, oLine)
        classify_process_end_keyword(dVars, oLine)


def classify_process_keyword(dVars, oLine):
    if re.match('^\s*process', oLine.lineLower):
        oLine.isProcessKeyword = True
        oLine.insideProcess = True
        oLine.indentLevel = dVars['iCurrentIndentLevel']
    if re.match('^\s*\S+\s*:\s*process', oLine.lineLower):
        oLine.isProcessKeyword = True
        oLine.insideProcess = True
        oLine.indentLevel = dVars['iCurrentIndentLevel']
        oLine.isProcessLabel = True


def classify_process_sensitivity_list(dVars, oLine):
    # Check sensitivity list
    if '(' in oLine.line and not oLine.insideSensitivityList and not dVars['fFoundProcessBegin'] and not dVars['SensitivityListFound']:
        oLine.isSensitivityListBegin = True
        oLine.insideSensitivityList = True
        dVars['SensitivityListFound'] = True
    if oLine.insideSensitivityList:
        dVars['iOpenParenthesis'] += oLine.line.count('(')
        dVars['iCloseParenthesis'] += oLine.line.count(')')
        if dVars['iOpenParenthesis'] == dVars['iCloseParenthesis']:
            oLine.isSensitivityListEnd = True
            dVars['iOpenParenthesis'] = 0
            dVars['iCloseParenthesis'] = 0
            dVars['iCurrentIndentLevel'] += 1


def classify_process_begin_keyword(dVars, oLine):
    if re.match('^.*\s+begin', oLine.lineLower) or re.match('^\s*begin', oLine.lineLower):
        oLine.indentLevel = dVars['iCurrentIndentLevel'] - 1
        oLine.isProcessBegin = True
        dVars['fFoundProcessBegin'] = True


def classify_process_end_keyword(dVars, oLine):
    if re.match('^\s*end\s+process', oLine.lineLower):
        oLine.indentLevel = dVars['iCurrentIndentLevel'] - 1
        oLine.isEndProcess = True
        dVars['fFoundProcessBegin'] = False
        dVars['iCurrentIndentLevel'] = dVars['iCurrentIndentLevel'] - 1
        dVars['SensitivityListFound'] = False



import os

import unittest
from vsg import vhdlFile

oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','if_statement','if_test_input.vhd'))
oFileCompress = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','if_statement','if_compressed_line_test_input.vhd'))

class testVhdlFileIfAssignments(unittest.TestCase):


    def test_insideIf_assignment(self):
        lExpected = [8,13,14,19,20,21,24,30,31,32,33,41,42,43,46,52,53,54,57,66,67,68,73,80,91,98,99,100,105,107,108,109,112,114,115]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.insideIf:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isElseIfKeyword_assignment(self):
        lExpected = [24,73,99,100,108,114]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isElseIfKeyword:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isEndIfKeyword_assignment(self):
        lExpected = [11,17,27,36,39,49,50,60,62,78,89,96,103,105,110,117]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isEndIfKeyword:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isIfKeyword_assignment(self):
        lExpected = [8,13,19,30,33,41,46,52,57,66,80,91,98,105,107,112]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isIfKeyword:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isThenKeyword_assignment(self):
        lExpected = [8,14,21,24,32,33,43,46,54,57,68,73,80,91,98,99,100,105,107,109,112,115]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isThenKeyword:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isElseKeyword_assignment(self):
        lExpected = [85,94,102,105,109]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isElseKeyword:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_ifIndent_assignment(self):
        self.maxDiff = None
#       lExpected = [   0,   1,2,   3,4,   5,6,7,8,9,10,11,  12,13,  14,15,16,17,  18,19,  20,  21,22,23,24,25,26,27,  28,  29]
        lExpected = [None,None,0,None,0,None,1,1,2,3, 3, 2,None, 2,None, 3, 3, 2,None, 2,None,None, 3, 3, 2, 3, 3, 2,None,None]
        # Generic actual list
        lActual = []
        iMaxCheck = len(lExpected)
        for iIndex, oLine in enumerate(oFile.lines):
            if iIndex == iMaxCheck:
                break
            lActual.append(oLine.indentLevel)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_compressed_line_assignment(self):
        self.assertTrue(oFileCompress.lines[10].insideIf)
        self.assertTrue(oFileCompress.lines[10].isElseKeyword)
        self.assertTrue(oFileCompress.lines[10].isElseIfKeyword)
        self.assertTrue(oFileCompress.lines[10].isEndIfKeyword)
        self.assertTrue(oFileCompress.lines[10].isIfKeyword)
        self.assertTrue(oFileCompress.lines[10].isThenKeyword)

        self.assertTrue(oFileCompress.lines[12].insideIf)
        self.assertTrue(oFileCompress.lines[12].isElseKeyword)
        self.assertTrue(oFileCompress.lines[12].isElseIfKeyword)
        self.assertTrue(oFileCompress.lines[12].isEndIfKeyword)
        self.assertTrue(oFileCompress.lines[12].isIfKeyword)
        self.assertTrue(oFileCompress.lines[12].isThenKeyword)



import os

import unittest

from vsg.rules import assert_statement
from vsg import vhdlFile


# Read in test file used for all tests
oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'assert_test_input.vhd'))

class testRuleAssertMethods(unittest.TestCase):

    def test_rule_001(self):
        oRule = assert_statement.rule_001()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'assert')
        self.assertEqual(oRule.identifier, '001')

        dExpected = [10,16,21]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)



import os

import unittest
from vsg import vhdlFile

oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','generate','generate_test_input.vhd'))

class testVhdlFileGenerateAssignments(unittest.TestCase):


    def test_isGenerateKeyword(self):
        lExpected = [6,11,16,21,26,31,36,41,46,51,56,60,65,68,71,77,83]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isGenerateKeyword:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isGenerateBegin(self):
        lExpected = [7,12,17,22,27,32,37,42,47,52,84]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isGenerateBegin:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isGenerateEnd(self):
        lExpected = [9,14,19,24,29,34,39,44,49,54,58,62,73,75,79,81,86]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isGenerateEnd:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isEndArchitecture(self):
        lExpected = [89]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isEndArchitecture:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_GenerateIndent(self):
        #           [   0,   1,2,   3,4,   5,6,7,8,9,  10,11,12,13,14,  15]
        lExpected = [None,None,0,None,0,None,1,1,2,1,None, 1, 1, 2, 1,None]
        # Generic actual list
        lActual = []
        iMaxCheck = len(lExpected)
        for iIndex, oLine in enumerate(oFile.lines):
            if iIndex == iMaxCheck:
                break
            lActual.append(oLine.indentLevel)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_insideGenerate(self):
        lExpected = [6,7,8,9,11,12,13,14,16,17,18,19,21,22,23,24,26,27,28,29,31,32,33,34,36,37,38,39,41,42,43,44,46,47,48,49,51,52,53,54,56,57,58,60,61,62,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,83,84,85,86]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.insideGenerate:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)




import os

import unittest

from vsg.rules import entity
from vsg import vhdlFile


oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','entity','entity_test_input.vhd'))


class testFixRuleEntityMethods(unittest.TestCase):

    def test_fix_rule_001(self):
        oRule = entity.rule_001()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_002(self):
        oRule = entity.rule_002()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_003(self):
        oRule = entity.rule_003()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_004(self):
        oRule = entity.rule_004()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_005(self):
        oRule = entity.rule_005()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])
        self.assertEqual(oFile.lines[51].line, '   generic  (')

    def test_fix_rule_006(self):
        oRule = entity.rule_006()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_007(self):
        oRule = entity.rule_007()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_008(self):
        oRule = entity.rule_008()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_009(self):
        oRule = entity.rule_009()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_010(self):
        oRule = entity.rule_010()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_011(self):
        oRule = entity.rule_011()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_012(self):
        oRule = entity.rule_012()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_013(self):
        oRule = entity.rule_013()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_014(self):
        oRule = entity.rule_014()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_015(self):
        oRule = entity.rule_015()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])
        self.assertEqual(oFile.lines[103].line, 'end entity ENT1;')

    def test_fix_rule_016(self):
        oRule = entity.rule_016()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_017(self):
        oRule = entity.rule_017()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_018(self):
        oRule = entity.rule_018()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])
        self.assertEqual(oFile.lines[21].line, '       G_GENERIC1 : std_logic :=\'0\';          -- Comment')
        self.assertEqual(oFile.lines[22].line, '    G_generic2 : std_logic := \'1\'             -- Comment')



import os

import unittest

from vsg.rules import port
from vsg import vhdlFile

oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','port','port_test_input.vhd'))
oFile_rule_016 = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','port','port_test_input.vhd'))


class testFixRulePortMethods(unittest.TestCase):

    def test_fix_rule_001(self):
        oRule = port.rule_001()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_002(self):
        oRule = port.rule_002()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_003(self):
        oRule = port.rule_003()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_004(self):
        oRule = port.rule_004()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_005(self):
        oRule = port.rule_005()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_006(self):
        oRule = port.rule_006()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_007(self):
        oRule = port.rule_007()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_008(self):
        oRule = port.rule_008()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_009(self):
        oRule = port.rule_009()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_010(self):
        oRule = port.rule_010()
        oRule.fix(oFile)
        self.assertEqual(oFile.lines[27].line, '    IO_PORT3(c_index) : inout NATURAL;')
        self.assertEqual(oFile.lines[28].line, '    PORT4(c_index) : in    INTEGER;')
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

#### RULE 11 and RULE 12 will be fixed as they would modify code.
#    def test_fix_rule_011_prefix(self):
#        oRule = port.rule_011()
#        oRule.fix(oFile)
#        dExpected = [12,13,14,29,30,31,43,44,45,60,61,62,74,75,76,99,100,101]
#        oRule.anal[](oFile)
#        self.assertEqual(oRule.violations, dExpected)
#
#    def test_fix_rule_011_none(self):
#        oRule = port.rule_011()
#        oRule.fix(oFile)
#
#    def test_fix_rule_011_suf[](self):
#        oRule = port.rule_011()
#        oRule.port_direction = 'Suffix'
#
#        dExpected = [9,10,11,12,13,14,26,27,28,29,30,31,40,41,42,43,44,45,57,58,59,60,61,62,71,72,73,74,75,76,87,88,89,119,120,121,129,130,131,141,142]
#        oRule.analyze(oFile)
#        self.assertEqual(oRule.violations, dExpected)

#    def test_fix_rule_012(self):
#        oRule = port.rule_012()
#        oRule.fix(oFile)
#        oRule.analyze(oFile)
#        self.assertEqual(oRule.violations, [])

    def test_fix_rule_013(self):
        oRule = port.rule_013()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_014(self):
        oRule = port.rule_014()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])
        self.assertFalse(oFile.lines[30].isEndPortMap)
        self.assertTrue(oFile.lines[30].isPortDeclaration)
        self.assertEqual(oFile.lines[30].indentLevel, oFile.lines[29].indentLevel)
        self.assertTrue(oFile.lines[31].insidePortMap)
        self.assertTrue(oFile.lines[31].isEndPortMap)
        self.assertEqual(oFile.lines[31].insideEntity, oFile.lines[30].insideEntity)
        self.assertFalse(oFile.lines[31].isPortDeclaration)
        self.assertEqual(oFile.lines[31].indentLevel, oFile.lines[29].indentLevel - 1)

    def test_fix_rule_015(self):
        oRule = port.rule_015()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_016(self):
        oRule = port.rule_016()
        oRule.fix(oFile_rule_016)
        oRule.analyze(oFile_rule_016)
        self.assertEqual(oRule.violations, [])
        self.assertEqual(oFile_rule_016.lines[140].indentLevel + 1, oFile_rule_016.lines[141].indentLevel)

    def test_fix_rule_017(self):
        oRule = port.rule_017()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_018(self):
        oRule = port.rule_018()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_019(self):
        oRule = port.rule_019()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_020(self):
        oRule = port.rule_020()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])

    def test_fix_rule_021(self):
        oRule = port.rule_021()
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, [])
        self.assertEqual(oFile.lines[152].line, '  port (')
        self.assertEqual(oFile.lines[153].line, '    PORT1 : in    std_logic')



import os

import unittest

from vsg.rules import while_loop
from vsg import vhdlFile

# Read in test file used for all tests
oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','while_loop','while_loop_test_input.vhd'))

class testRuleWhileLoopMethods(unittest.TestCase):

    def test_rule_001(self):
        oRule = while_loop.rule_001()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'while_loop')
        self.assertEqual(oRule.identifier, '001')
        dExpected = [13]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_002(self):
        oRule = while_loop.rule_002()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'while_loop')
        self.assertEqual(oRule.identifier, '002')
        dExpected = [19]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)




from vsg.rules import indent_rule


class rule_006(indent_rule):
    '''
    Process rule 006 checks for the proper indentation at the beginning of the line.
    '''

    def __init__(self):
        indent_rule.__init__(self, 'process', '006', 'isEndProcess')



import os

import unittest
from vsg import vhdlFile

oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','package_body','package_body_test_input.vhd'))

class testVhdlFilePackageBodyMethods(unittest.TestCase):


    def test_insidePackageBody_assignment(self):
        lExpected = [7,8,9]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.insidePackageBody:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isPackageBodyKeyword_assignment(self):
        lExpected = [7]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isPackageBodyKeyword:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_isPackageBodyEnd_assignment(self):
        lExpected = [9]
        # Generic actual list
        lActual = []
        for iIndex, oLine in enumerate(oFile.lines):
            if oLine.isPackageBodyEnd:
                lActual.append(iIndex)
        # Compare
        self.assertEqual(lActual, lExpected)

    def test_package_bodyIndent_assignment(self):
#        lExpected = [7,   8,9]
        lExpected =  [0,None,0]
        # Generic actual list
        lActual = []
        for oLine in oFile.lines:
            if oLine.insidePackageBody:
                lActual.append(oLine.indentLevel)
        # Compare
        self.assertEqual(lActual, lExpected)
